declare const IteratorSlot: unique symbol;
declare type Parameters<T, R> = T extends (...args: infer T) => R ? T : never;
export declare function wrap<It, T, R>(func: (...args: Parameters<T, R>) => R): (this: any, ...args: any) => HIterator<It>;
export declare function awrap<It, T, R>(func: (...args: Parameters<T, R>) => R): (this: any, ...args: any) => HAsyncIterator<It>;
export declare function aiter<T>(item: AsyncIterator<T> | AsyncIterator<T>): HAsyncIterator<T>;
export declare class HAsyncIterator<T> implements AsyncIterator<T> {
    [IteratorSlot]: AsyncIterator<T>;
    constructor(iterator?: AsyncIterator<T>);
    next(val?: any): Promise<IteratorResult<T, any>>;
    throw(val?: any): Promise<IteratorResult<T, any>>;
    return(val?: any): Promise<IteratorResult<T, any>>;
    /** Map each value of iterator to another value via {callback}. */
    map<R>(callback: (value: T) => R | PromiseLike<R>): HAsyncIterator<R>;
    static map<T, R>(this: AsyncIterator<T>, callback: (value: T) => R | PromiseLike<R>): AsyncGenerator<R, any, unknown>;
    /** Each value is given through {callback}, return `true` if value is needed into returned iterator. */
    filter(callback: (value: T) => boolean | PromiseLike<boolean>): HAsyncIterator<T>;
    static filter<T>(this: AsyncIterator<T>, callback: (value: T) => boolean | PromiseLike<boolean>): AsyncGenerator<T, any, any>;
    /** Find a specific value that returns `true` in {callback}, and return it. Returns `undefined` otherwise. */
    find(callback: (value: T) => boolean | PromiseLike<boolean>): Promise<T | undefined>;
    /** Return `true` if each value of iterator validate {callback}. */
    every(callback: (value: T) => boolean | PromiseLike<boolean>): Promise<boolean>;
    /** Return `true` if one value of iterator validate {callback}. */
    some(callback: (value: T) => boolean | PromiseLike<boolean>): Promise<boolean>;
    /** Consume iterator and collapse values inside an array. */
    toArray(max_count?: number): Promise<T[]>;
    /** Create a new iterator that consume {limit} items, then stops. */
    take(limit: number): HAsyncIterator<T>;
    static take<T>(this: AsyncIterator<T>, limit: number): AsyncGenerator<T, any, any>;
    /** Create a new iterator that skip {limit} items from source iterator, then yield all values. */
    drop(limit: number): HAsyncIterator<T>;
    static drop<T>(this: AsyncIterator<T>, limit: number): AsyncGenerator<T, any, any>;
    /** Get a pair [index, value] for each remaining value of iterable. */
    asIndexedPairs(): HAsyncIterator<[number, T]>;
    static asIndexedPairs<T>(this: AsyncIterator<T>): AsyncGenerator<(number | T)[], any, unknown>;
    /** Like map, but you can return a new iterator that will be flattened. */
    flatMap<R>(mapper: (value: T) => AsyncIterator<R> | R | PromiseLike<AsyncIterator<R>> | PromiseLike<R>): HAsyncIterator<R>;
    static flatMap<T, R>(this: AsyncIterator<T>, mapper: (value: T) => AsyncIterator<R> | R | PromiseLike<AsyncIterator<R>> | PromiseLike<R>): AsyncGenerator<any, any, any>;
    /** Accumulate each item inside **acc** for each value **value**. */
    reduce<V>(reducer: (acc: V, value: T) => V | PromiseLike<V>, initial_value?: V): Promise<V | undefined>;
    /** Iterate over each value of iterator by calling **callback** for each value. */
    forEach(callback: (value: T) => any | PromiseLike<any>): Promise<void>;
    /** Join all the remaining elements of the iterator in a single string with glue {glue}. */
    join(string: string): Promise<string>;
    /** End the iterator and return the number of remaining items. */
    count(): Promise<number>;
    /** Iterate through current iterator, then through the given iterators in the correct order. */
    chain<I>(...iterables: AsyncIterableIterator<I>[]): HAsyncIterator<T | I>;
    static chain<T, I>(this: AsyncIterableIterator<T>, ...iterables: AsyncIterableIterator<I>[]): AsyncGenerator<T | I, void, undefined>;
    /** Iterate through multiple iterators together. */
    zip<O>(...others: AsyncIterableIterator<O>[]): HAsyncIterator<(T | O)[]>;
    static zip<T, O>(this: AsyncIterableIterator<T>, ...others: AsyncIterableIterator<O>[]): AsyncIterator<(T | O)[]>;
    /** Continue iterator until {callback} return a falsy value. */
    takeWhile(callback: (value: T) => boolean | PromiseLike<boolean>): HAsyncIterator<T>;
    static takeWhile<T>(this: AsyncIterator<T>, callback: (value: T) => boolean | PromiseLike<boolean>): AsyncGenerator<T, any, any>;
    /** Skip elements until {callback} return a truthy value. */
    dropWhile(callback: (value: T) => boolean | PromiseLike<boolean>): HAsyncIterator<T>;
    static dropWhile<T>(this: AsyncIterator<T>, callback: (value: T) => boolean | PromiseLike<boolean>): AsyncGenerator<T, any, any>;
    /** Continue iterator until `null` or `undefined` is encountered. */
    fuse(): HAsyncIterator<T>;
    static fuse<T>(this: AsyncIterator<T>): AsyncGenerator<T, any, any>;
    /** Partition {true} elements to first array, {false} elements to second one. */
    partition(callback: (value: T) => boolean | PromiseLike<boolean>): Promise<T[][]>;
    /** Find the iterator index of the first element that returns a truthy value, -1 otherwise. */
    findIndex(callback: (value: T) => boolean | PromiseLike<boolean>): Promise<number>;
    /** Only works if it. is a number iterator. Returns the maximum of iterator. */
    max(): Promise<number>;
    /** Only works if it. is a number iterator. Returns the minimum of iterator. */
    min(): Promise<number>;
    /** When iterator ends, go back to the first item then loop. Indefinitively. */
    cycle(): HAsyncIterator<T>;
    static cycle<T>(this: AsyncIterator<T>): AsyncGenerator<T, void, unknown>;
    [Symbol.asyncIterator](): this;
}
export declare function iter<T>(item: Iterator<T> | Iterable<T>): HIterator<T>;
export declare class HIterator<T> implements Iterator<T> {
    [IteratorSlot]: Iterator<T>;
    constructor(iterator?: Iterator<T>);
    next(val?: any): IteratorResult<T, any>;
    throw(val?: any): IteratorResult<T, any>;
    return(val?: any): IteratorResult<T, any>;
    /** Map each value of iterator to another value via {callback}. */
    map<R>(callback: (element: T) => R): HIterator<T>;
    static map<T, R>(this: Iterator<T>, callback: (element: T) => R): Generator<R, any, unknown>;
    /** Each value is given through {callback}, return `true` if value is needed into returned iterator. */
    filter(callback: (value: T) => boolean): HIterator<T>;
    static filter<T>(this: Iterator<T>, callback: (value: T) => boolean): Generator<T, any, any>;
    /** Find a specific value that returns `true` in {callback}, and return it. Returns `undefined` otherwise. */
    find(callback: (value: T) => boolean): T | undefined;
    /** Return `true` if each value of iterator validate {callback}. */
    every(callback: (value: T) => boolean): boolean;
    /** Return `true` if one value of iterator validate {callback}. */
    some(callback: (value: T) => boolean): boolean;
    /** Consume iterator and collapse values inside an array. */
    toArray(max_count?: number): T[];
    /** Create a new iterator that consume {limit} items, then stops. */
    take(limit: number): HIterator<T>;
    static take<T>(this: Iterator<T>, limit: number): Generator<T, any, any>;
    /** Create a new iterator that skip {limit} items from source iterator, then yield all values. */
    drop(limit: number): HIterator<T>;
    static drop<T>(this: Iterator<T>, limit: number): Generator<T, any, any>;
    /** Get a pair [index, value] for each remaining value of iterable. */
    asIndexedPairs(): HIterator<[number, T]>;
    static asIndexedPairs<T>(this: Iterator<T>): Generator<(number | T)[], any, unknown>;
    /** Like map, but you can return a new iterator that will be flattened. */
    flatMap<R>(mapper: (value: T) => IterableIterator<R> | R): HIterator<R>;
    static flatMap<T, R>(this: Iterator<T>, mapper: (value: T) => IterableIterator<R> | R): Iterator<R>;
    /** Accumulate each item inside **acc** for each value **value**. */
    reduce<V>(reducer: (acc: V, value: T) => V, initial_value?: V): V | undefined;
    /** Iterate over each value of iterator by calling **callback** for each value. */
    forEach(callback: (value: T) => any): void;
    /** End the iterator and return the number of remaining items. */
    count(): number;
    /** Join all the remaining elements of the iterator in a single string with glue {glue}. */
    join(string: string): string;
    /** Iterate through current iterator, then through the given iterators in the correct order. */
    chain<I>(...iterables: IterableIterator<I>[]): HIterator<T | I>;
    static chain<T, I>(this: IterableIterator<T>, ...iterables: IterableIterator<I>[]): Generator<T | I, void, undefined>;
    /** Iterate through multiple iterators together. */
    zip<O>(...others: IterableIterator<O>[]): HIterator<(T | O)[]>;
    static zip<T, O>(this: Iterator<T>, ...others: IterableIterator<O>[]): Iterator<(T | O)[]>;
    /** Continue iterator until {callback} return a falsy value. */
    takeWhile(callback: (value: T) => boolean): HIterator<T>;
    static takeWhile<T>(this: Iterator<T>, callback: (value: T) => boolean): Generator<T, any, any>;
    /** Skip elements until {callback} return a truthy value. */
    dropWhile(callback: (value: T) => boolean): HIterator<T>;
    static dropWhile<T>(this: Iterator<T>, callback: (value: T) => boolean): Generator<T, any, any>;
    /** Continue iterator until `null` or `undefined` is encountered. */
    fuse(): HIterator<T>;
    static fuse<T>(this: Iterator<T>): Generator<T, any, any>;
    /** Partition {true} elements to first array, {false} elements to second one. */
    partition(callback: (value: T) => boolean): [T[], T[]];
    /** Find the iterator index of the first element that returns a truthy value, -1 otherwise. */
    findIndex(callback: (value: T) => boolean): number;
    /** Only works if it. is a number iterator. Returns the maximum of iterator. */
    max(): number;
    /** Only works if it. is a number iterator. Returns the minimum of iterator. */
    min(): number;
    /** When iterator ends, go back to the first item then loop. Indefinitively. */
    cycle(): HIterator<T>;
    static cycle<T>(this: Iterator<T>): Generator<T, void, undefined>;
    [Symbol.iterator](): this;
}
export {};
